@@ -17,9 +17,6 @@ from pyomo.common.config import ConfigBlock, ConfigValue, In, NonNegativeFloat,
 
 logger = logging.getLogger(__name__)
 
-if not hasattr(math, 'inf'):
-    math.inf = float('inf')
-
 
 """
 The purpose of this file is to perform feasibility based bounds 
@@ -103,6 +100,27 @@ def _prop_bnds_leaf_to_root_SumExpression(node, bnds_dict, feasibility_tol):
     bnds_dict[node] = (lb, ub)
 
 
+def _prop_bnds_leaf_to_root_DivisionExpression(node, bnds_dict, feasibility_tol):
+    """
+
+    Parameters
+    ----------
+    node: pyomo.core.expr.numeric_expr.DivisionExpression
+    bnds_dict: ComponentMap
+    feasibility_tol: float
+        If the bounds computed on the body of a constraint violate the bounds of the constraint by more than
+        feasibility_tol, then the constraint is considered infeasible and an exception is raised. This tolerance
+        is also used when performing certain interval arithmetic operations to ensure that none of the feasible
+        region is removed due to floating point arithmetic and to prevent math domain errors (a larger value
+        is more conservative).
+    """
+    assert len(node.args) == 2
+    arg1, arg2 = node.args
+    lb1, ub1 = bnds_dict[arg1]
+    lb2, ub2 = bnds_dict[arg2]
+    bnds_dict[node] = interval.div(lb1, ub1, lb2, ub2, feasibility_tol=feasibility_tol)
+
+
 def _prop_bnds_leaf_to_root_PowExpression(node, bnds_dict, feasibility_tol):
     """
 
@@ -204,6 +222,26 @@ def _prop_bnds_leaf_to_root_log(node, bnds_dict, feasibility_tol):
     bnds_dict[node] = interval.log(lb1, ub1)
 
 
+def _prop_bnds_leaf_to_root_log10(node, bnds_dict, feasibility_tol):
+    """
+
+    Parameters
+    ----------
+    node: pyomo.core.expr.numeric_expr.UnaryFunctionExpression
+    bnds_dict: ComponentMap
+    feasibility_tol: float
+        If the bounds computed on the body of a constraint violate the bounds of the constraint by more than
+        feasibility_tol, then the constraint is considered infeasible and an exception is raised. This tolerance
+        is also used when performing certain interval arithmetic operations to ensure that none of the feasible
+        region is removed due to floating point arithmetic and to prevent math domain errors (a larger value
+        is more conservative).
+    """
+    assert len(node.args) == 1
+    arg = node.args[0]
+    lb1, ub1 = bnds_dict[arg]
+    bnds_dict[node] = interval.log10(lb1, ub1)
+
+
 def _prop_bnds_leaf_to_root_sin(node, bnds_dict, feasibility_tol):
     """
 
@@ -281,7 +319,7 @@ def _prop_bnds_leaf_to_root_asin(node, bnds_dict, feasibility_tol):
     assert len(node.args) == 1
     arg = node.args[0]
     lb1, ub1 = bnds_dict[arg]
-    bnds_dict[node] = interval.asin(lb1, ub1, -math.inf, math.inf)
+    bnds_dict[node] = interval.asin(lb1, ub1, -interval.inf, interval.inf)
 
 
 def _prop_bnds_leaf_to_root_acos(node, bnds_dict, feasibility_tol):
@@ -301,7 +339,7 @@ def _prop_bnds_leaf_to_root_acos(node, bnds_dict, feasibility_tol):
     assert len(node.args) == 1
     arg = node.args[0]
     lb1, ub1 = bnds_dict[arg]
-    bnds_dict[node] = interval.acos(lb1, ub1, -math.inf, math.inf)
+    bnds_dict[node] = interval.acos(lb1, ub1, -interval.inf, interval.inf)
 
 
 def _prop_bnds_leaf_to_root_atan(node, bnds_dict, feasibility_tol):
@@ -321,7 +359,7 @@ def _prop_bnds_leaf_to_root_atan(node, bnds_dict, feasibility_tol):
     assert len(node.args) == 1
     arg = node.args[0]
     lb1, ub1 = bnds_dict[arg]
-    bnds_dict[node] = interval.atan(lb1, ub1, -math.inf, math.inf)
+    bnds_dict[node] = interval.atan(lb1, ub1, -interval.inf, interval.inf)
 
 
 def _prop_bnds_leaf_to_root_sqrt(node, bnds_dict, feasibility_tol):
@@ -347,6 +385,7 @@ def _prop_bnds_leaf_to_root_sqrt(node, bnds_dict, feasibility_tol):
 _unary_leaf_to_root_map = dict()
 _unary_leaf_to_root_map['exp'] = _prop_bnds_leaf_to_root_exp
 _unary_leaf_to_root_map['log'] = _prop_bnds_leaf_to_root_log
+_unary_leaf_to_root_map['log10'] = _prop_bnds_leaf_to_root_log10
 _unary_leaf_to_root_map['sin'] = _prop_bnds_leaf_to_root_sin
 _unary_leaf_to_root_map['cos'] = _prop_bnds_leaf_to_root_cos
 _unary_leaf_to_root_map['tan'] = _prop_bnds_leaf_to_root_tan
@@ -373,7 +412,7 @@ def _prop_bnds_leaf_to_root_UnaryFunctionExpression(node, bnds_dict, feasibility
     if node.getname() in _unary_leaf_to_root_map:
         _unary_leaf_to_root_map[node.getname()](node, bnds_dict, feasibility_tol)
     else:
-        bnds_dict[node] = (-math.inf, math.inf)
+        bnds_dict[node] = (-interval.inf, interval.inf)
 
 
 def _prop_bnds_leaf_to_root_GeneralExpression(node, bnds_dict, feasibility_tol):
@@ -397,6 +436,7 @@ def _prop_bnds_leaf_to_root_GeneralExpression(node, bnds_dict, feasibility_tol):
 
 _prop_bnds_leaf_to_root_map = dict()
 _prop_bnds_leaf_to_root_map[numeric_expr.ProductExpression] = _prop_bnds_leaf_to_root_ProductExpression
+_prop_bnds_leaf_to_root_map[numeric_expr.DivisionExpression] = _prop_bnds_leaf_to_root_DivisionExpression
 _prop_bnds_leaf_to_root_map[numeric_expr.ReciprocalExpression] = _prop_bnds_leaf_to_root_ReciprocalExpression
 _prop_bnds_leaf_to_root_map[numeric_expr.PowExpression] = _prop_bnds_leaf_to_root_PowExpression
 _prop_bnds_leaf_to_root_map[numeric_expr.SumExpression] = _prop_bnds_leaf_to_root_SumExpression
@@ -405,6 +445,7 @@ _prop_bnds_leaf_to_root_map[numeric_expr.NegationExpression] = _prop_bnds_leaf_t
 _prop_bnds_leaf_to_root_map[numeric_expr.UnaryFunctionExpression] = _prop_bnds_leaf_to_root_UnaryFunctionExpression
 
 _prop_bnds_leaf_to_root_map[numeric_expr.NPV_ProductExpression] = _prop_bnds_leaf_to_root_ProductExpression
+_prop_bnds_leaf_to_root_map[numeric_expr.NPV_DivisionExpression] = _prop_bnds_leaf_to_root_DivisionExpression
 _prop_bnds_leaf_to_root_map[numeric_expr.NPV_ReciprocalExpression] = _prop_bnds_leaf_to_root_ReciprocalExpression
 _prop_bnds_leaf_to_root_map[numeric_expr.NPV_PowExpression] = _prop_bnds_leaf_to_root_PowExpression
 _prop_bnds_leaf_to_root_map[numeric_expr.NPV_SumExpression] = _prop_bnds_leaf_to_root_SumExpression
@@ -523,12 +564,45 @@ def _prop_bnds_root_to_leaf_SumExpression(node, bnds_dict, feasibility_tol):
     bnds_dict[node.arg(0)] = (lb, ub)
 
 
+def _prop_bnds_root_to_leaf_DivisionExpression(node, bnds_dict, feasibility_tol):
+    """
+
+    Parameters
+    ----------
+    node: pyomo.core.expr.numeric_expr.DivisionExpression
+    bnds_dict: ComponentMap
+    feasibility_tol: float
+        If the bounds computed on the body of a constraint violate the bounds of the constraint by more than
+        feasibility_tol, then the constraint is considered infeasible and an exception is raised. This tolerance
+        is also used when performing certain interval arithmetic operations to ensure that none of the feasible
+        region is removed due to floating point arithmetic and to prevent math domain errors (a larger value
+        is more conservative).
+    """
+    assert len(node.args) == 2
+    arg1, arg2 = node.args
+    lb0, ub0 = bnds_dict[node]
+    lb1, ub1 = bnds_dict[arg1]
+    lb2, ub2 = bnds_dict[arg2]
+    _lb1, _ub1 = interval.mul(lb0, ub0, lb2, ub2)
+    _lb2, _ub2 = interval.div(lb1, ub1, lb0, ub0, feasibility_tol=feasibility_tol)
+    if _lb1 > lb1:
+        lb1 = _lb1
+    if _ub1 < ub1:
+        ub1 = _ub1
+    if _lb2 > lb2:
+        lb2 = _lb2
+    if _ub2 < ub2:
+        ub2 = _ub2
+    bnds_dict[arg1] = (lb1, ub1)
+    bnds_dict[arg2] = (lb2, ub2)
+
+
 def _prop_bnds_root_to_leaf_PowExpression(node, bnds_dict, feasibility_tol):
     """
 
     Parameters
     ----------
-    node: pyomo.core.expr.numeric_expr.ProductExpression
+    node: pyomo.core.expr.numeric_expr.PowExpression
     bnds_dict: ComponentMap
     feasibility_tol: float
         If the bounds computed on the body of a constraint violate the bounds of the constraint by more than
@@ -592,7 +666,7 @@ def _prop_bnds_root_to_leaf_ReciprocalExpression(node, bnds_dict, feasibility_to
 
     Parameters
     ----------
-    node: pyomo.core.expr.numeric_expr.ProductExpression
+    node: pyomo.core.expr.numeric_expr.ReciprocalExpression
     bnds_dict: ComponentMap
     feasibility_tol: float
         If the bounds computed on the body of a constraint violate the bounds of the constraint by more than
@@ -618,7 +692,7 @@ def _prop_bnds_root_to_leaf_NegationExpression(node, bnds_dict, feasibility_tol)
 
     Parameters
     ----------
-    node: pyomo.core.expr.numeric_expr.ProductExpression
+    node: pyomo.core.expr.numeric_expr.NegationExpression
     bnds_dict: ComponentMap
     feasibility_tol: float
         If the bounds computed on the body of a constraint violate the bounds of the constraint by more than
@@ -691,6 +765,32 @@ def _prop_bnds_root_to_leaf_log(node, bnds_dict, feasibility_tol):
     bnds_dict[arg] = (lb1, ub1)
 
 
+def _prop_bnds_root_to_leaf_log10(node, bnds_dict, feasibility_tol):
+    """
+
+    Parameters
+    ----------
+    node: pyomo.core.expr.numeric_expr.ProductExpression
+    bnds_dict: ComponentMap
+    feasibility_tol: float
+        If the bounds computed on the body of a constraint violate the bounds of the constraint by more than
+        feasibility_tol, then the constraint is considered infeasible and an exception is raised. This tolerance
+        is also used when performing certain interval arithmetic operations to ensure that none of the feasible
+        region is removed due to floating point arithmetic and to prevent math domain errors (a larger value
+        is more conservative).
+    """
+    assert len(node.args) == 1
+    arg = node.args[0]
+    lb0, ub0 = bnds_dict[node]
+    lb1, ub1 = bnds_dict[arg]
+    _lb1, _ub1 = interval.power(10, 10, lb0, ub0)
+    if _lb1 > lb1:
+        lb1 = _lb1
+    if _ub1 < ub1:
+        ub1 = _ub1
+    bnds_dict[arg] = (lb1, ub1)
+
+
 def _prop_bnds_root_to_leaf_sin(node, bnds_dict, feasibility_tol):
     """
 
@@ -850,6 +950,7 @@ def _prop_bnds_root_to_leaf_atan(node, bnds_dict, feasibility_tol):
 _unary_root_to_leaf_map = dict()
 _unary_root_to_leaf_map['exp'] = _prop_bnds_root_to_leaf_exp
 _unary_root_to_leaf_map['log'] = _prop_bnds_root_to_leaf_log
+_unary_root_to_leaf_map['log10'] = _prop_bnds_root_to_leaf_log10
 _unary_root_to_leaf_map['sin'] = _prop_bnds_root_to_leaf_sin
 _unary_root_to_leaf_map['cos'] = _prop_bnds_root_to_leaf_cos
 _unary_root_to_leaf_map['tan'] = _prop_bnds_root_to_leaf_tan
@@ -902,6 +1003,7 @@ def _prop_bnds_root_to_leaf_GeneralExpression(node, bnds_dict, feasibility_tol):
 
 _prop_bnds_root_to_leaf_map = dict()
 _prop_bnds_root_to_leaf_map[numeric_expr.ProductExpression] = _prop_bnds_root_to_leaf_ProductExpression
+_prop_bnds_root_to_leaf_map[numeric_expr.DivisionExpression] = _prop_bnds_root_to_leaf_DivisionExpression
 _prop_bnds_root_to_leaf_map[numeric_expr.ReciprocalExpression] = _prop_bnds_root_to_leaf_ReciprocalExpression
 _prop_bnds_root_to_leaf_map[numeric_expr.PowExpression] = _prop_bnds_root_to_leaf_PowExpression
 _prop_bnds_root_to_leaf_map[numeric_expr.SumExpression] = _prop_bnds_root_to_leaf_SumExpression
@@ -910,6 +1012,7 @@ _prop_bnds_root_to_leaf_map[numeric_expr.NegationExpression] = _prop_bnds_root_t
 _prop_bnds_root_to_leaf_map[numeric_expr.UnaryFunctionExpression] = _prop_bnds_root_to_leaf_UnaryFunctionExpression
 
 _prop_bnds_root_to_leaf_map[numeric_expr.NPV_ProductExpression] = _prop_bnds_root_to_leaf_ProductExpression
+_prop_bnds_root_to_leaf_map[numeric_expr.NPV_DivisionExpression] = _prop_bnds_root_to_leaf_DivisionExpression
 _prop_bnds_root_to_leaf_map[numeric_expr.NPV_ReciprocalExpression] = _prop_bnds_root_to_leaf_ReciprocalExpression
 _prop_bnds_root_to_leaf_map[numeric_expr.NPV_PowExpression] = _prop_bnds_root_to_leaf_PowExpression
 _prop_bnds_root_to_leaf_map[numeric_expr.NPV_SumExpression] = _prop_bnds_root_to_leaf_SumExpression
@@ -920,6 +1023,23 @@ _prop_bnds_root_to_leaf_map[_GeneralExpressionData] = _prop_bnds_root_to_leaf_Ge
 _prop_bnds_root_to_leaf_map[SimpleExpression] = _prop_bnds_root_to_leaf_GeneralExpression
 
 
+def _check_and_reset_bounds(var, lb, ub):
+    """
+    This function ensures that lb is not less than var.lb and that ub is not greater than var.ub.
+    """
+    orig_lb = value(var.lb)
+    orig_ub = value(var.ub)
+    if orig_lb is None:
+        orig_lb = -interval.inf
+    if orig_ub is None:
+        orig_ub = interval.inf
+    if lb < orig_lb:
+        lb = orig_lb
+    if ub > orig_ub:
+        ub = orig_ub
+    return lb, ub
+
+
 class _FBBTVisitorLeafToRoot(ExpressionValueVisitor):
     """
     This walker propagates bounds from the variables to each node in
@@ -946,7 +1066,7 @@ class _FBBTVisitorLeafToRoot(ExpressionValueVisitor):
         if node.__class__ in _prop_bnds_leaf_to_root_map:
             _prop_bnds_leaf_to_root_map[node.__class__](node, self.bnds_dict, self.feasibility_tol)
         else:
-            self.bnds_dict[node] = (-math.inf, math.inf)
+            self.bnds_dict[node] = (-interval.inf, interval.inf)
         return None
 
     def visiting_potential_leaf(self, node):
@@ -962,9 +1082,9 @@ class _FBBTVisitorLeafToRoot(ExpressionValueVisitor):
                 lb = value(node.lb)
                 ub = value(node.ub)
                 if lb is None:
-                    lb = -math.inf
+                    lb = -interval.inf
                 if ub is None:
-                    ub = math.inf
+                    ub = interval.inf
                 if lb - self.feasibility_tol > ub:
                     raise InfeasibleConstraintException('Variable has a lower bound which is larger than its upper bound: {0}'.format(str(node)))
             self.bnds_dict[node] = (lb, ub)
@@ -1017,6 +1137,27 @@ class _FBBTVisitorRootToLeaf(ExpressionValueVisitor):
         if node.is_variable_type():
             lb, ub = self.bnds_dict[node]
 
+            lb, ub = self.bnds_dict[node]
+            if lb > ub:
+                if lb - self.feasibility_tol > ub:
+                    raise InfeasibleConstraintException('Lower bound ({1}) computed for variable {0} is larger than the computed upper bound ({2}).'.format(node, lb, ub))
+                else:
+                    """
+                    If we reach this code, then lb > ub, but not by more than feasibility_tol. 
+                    Now we want to decrease lb slightly and increase ub slightly so that lb <= ub.
+                    However, we also have to make sure we do not make lb lower than the original lower bound
+                    and make sure we do not make ub larger than the original upper bound. This is what 
+                    _check_and_reset_bounds is for.
+                    """
+                    lb -= self.feasibility_tol
+                    ub += self.feasibility_tol
+                    lb, ub = _check_and_reset_bounds(node, lb, ub)
+                    self.bnds_dict[node] = (lb, ub)
+            if lb == interval.inf:
+                raise InfeasibleConstraintException('Computed a lower bound of +inf for variable {0}'.format(node))
+            if ub == -interval.inf:
+                raise InfeasibleConstraintException('Computed an upper bound of -inf for variable {0}'.format(node))
+
             if node.is_binary() or node.is_integer():
                 """
                 This bit of code has two purposes:
@@ -1027,24 +1168,21 @@ class _FBBTVisitorRootToLeaf(ExpressionValueVisitor):
                    and may give the wrong solution. Even if the correct solution is found, this could 
                    introduce numerical problems.
                 """
-                lb = max(math.floor(lb), math.ceil(lb - self.integer_tol))
-                ub = min(math.ceil(ub), math.floor(ub + self.integer_tol))
-                if lb < value(node.lb):
-                    lb = value(node.lb)  # don't make the bounds worse than the original bounds
-                if ub > value(node.ub):
-                    ub = value(node.ub)  # don't make the bounds worse than the original bounds
+                if lb > -interval.inf:
+                    lb = max(math.floor(lb), math.ceil(lb - self.integer_tol))
+                if ub < interval.inf:
+                    ub = min(math.ceil(ub), math.floor(ub + self.integer_tol))
+                """
+                We have to make sure we do not make lb lower than the original lower bound
+                and make sure we do not make ub larger than the original upper bound. This is what 
+                _check_and_reset_bounds is for.
+                """
+                lb, ub = _check_and_reset_bounds(node, lb, ub)
                 self.bnds_dict[node] = (lb, ub)
 
-            lb, ub = self.bnds_dict[node]
-            if lb - self.feasibility_tol > ub:
-                raise InfeasibleConstraintException('Lower bound computed for variable {0} is larger than the computed upper bound.'.format(node))
-            if lb == math.inf:
-                raise InfeasibleConstraintException('Computed a lower bound of +inf for variable {0}'.format(node))
-            if ub == -math.inf:
-                raise InfeasibleConstraintException('Computed an upper bound of -inf for variable {0}'.format(node))
-            if lb != -math.inf:
+            if lb != -interval.inf:
                 node.setlb(lb)
-            if ub != math.inf:
+            if ub != interval.inf:
                 node.setub(ub)
             return True, None
 
@@ -1111,9 +1249,9 @@ def _fbbt_con(con, config):
     _lb = value(con.lower)
     _ub = value(con.upper)
     if _lb is None:
-        _lb = -math.inf
+        _lb = -interval.inf
     if _ub is None:
-        _ub = math.inf
+        _ub = interval.inf
 
     lb, ub = bnds_dict[con.body]
 
@@ -1142,9 +1280,9 @@ def _fbbt_con(con, config):
             continue
         if _node.is_variable_type():
             lb, ub = bnds_dict[_node]
-            if lb == -math.inf:
+            if lb == -interval.inf:
                 lb = None
-            if ub == math.inf:
+            if ub == interval.inf:
                 ub = None
             new_var_bounds[_node] = (lb, ub)
     return new_var_bounds
@@ -1184,11 +1322,11 @@ def _fbbt_block(m, config):
             if v not in var_to_con_map:
                 var_to_con_map[v] = list()
             if v.lb is None:
-                var_lbs[v] = -math.inf
+                var_lbs[v] = -interval.inf
             else:
                 var_lbs[v] = value(v.lb)
             if v.ub is None:
-                var_ubs[v] = math.inf
+                var_ubs[v] = interval.inf
             else:
                 var_ubs[v] = value(v.ub)
             var_to_con_map[v].append(c)
