@@ -24,6 +24,7 @@ topdir = dirname(dirname(dirname(dirname(dirname(abspath(__file__))))))
 examplesdir = join(topdir, "examples", "kernel")
 
 examples = glob.glob(join(examplesdir,"*.py"))
+examples.extend(glob.glob(join(examplesdir,"mosek","*.py")))
 
 numpy_available = False
 try:
@@ -46,6 +47,7 @@ except:
 testing_solvers = {}
 testing_solvers['ipopt','nl'] = False
 testing_solvers['glpk','lp'] = False
+testing_solvers['mosek','python'] = False
 def setUpModule():
     global testing_solvers
     import pyomo.environ
@@ -69,6 +71,10 @@ def create_test_method(example):
                (not testing_solvers['ipopt','nl']) or \
                (not testing_solvers['glpk','lp']):
                 self.skipTest("Numpy or Scipy or Ipopt or Glpk is not available")
+        elif "mosek" in example:
+            if (not testing_solvers['ipopt','nl']) or \
+               (not testing_solvers['mosek','python']):
+                self.skipTest("Ipopt or Mosek is not available")
         rc, log = pyutilib.subprocess.run(['python',example])
         self.assertEqual(rc, 0, msg=log)
     return testmethod
