@@ -218,9 +218,13 @@ class TestPyomoUnit(unittest.TestCase):
         # test MonomialTermExpression
         self._get_check_units_ok(model.x*kg, uc, 'kg', expr.MonomialTermExpression)
 
-        # test ReciprocalExpression, NPV_ReciprocalExpression
-        self._get_check_units_ok(1.0/(model.x*kg), uc, '1 / kg', expr.ReciprocalExpression)
-        self._get_check_units_ok(1.0/kg, uc, '1 / kg', expr.NPV_ReciprocalExpression)
+        # test DivisionExpression, NPV_DivisionExpression
+        self._get_check_units_ok(1.0/(model.x*kg), uc, '1 / kg', expr.DivisionExpression)
+        self._get_check_units_ok(2.0/kg, uc, '1 / kg', expr.NPV_DivisionExpression)
+        self._get_check_units_ok((model.x*kg)/1.0, uc, 'kg', expr.MonomialTermExpression)
+        self._get_check_units_ok(kg/2.0, uc, 'kg', expr.NPV_DivisionExpression)
+        self._get_check_units_ok(model.y*m/(model.x*kg), uc, 'm / kg', expr.DivisionExpression)
+        self._get_check_units_ok(m/kg, uc, 'm / kg', expr.NPV_DivisionExpression)
         # I don't think that there are combinations that can "fail" for products
 
         # test PowExpression, NPV_PowExpression
