@@ -21,7 +21,7 @@ from operator import itemgetter
 import os
 thisfile = os.path.abspath(__file__)
 
-import pyomo.util.plugin
+import pyomo.common.plugin
 from pyomo.core import *
 from pyomo.core.base.var import _VarData
 from pyomo.core.base.piecewise import _PiecewiseData
@@ -146,7 +146,7 @@ class DDSIP_Input(object):
         scenario_instance = self._reference_scenario_instance
 
         # This is usuall called just prior to solving the instances,
-        # however the scenariotree uses CanonicalRepn for determining
+        # however the scenariotree uses StandardRepn for determining
         # constraint stage
         ph._preprocess_scenario_instances()
 
@@ -336,7 +336,8 @@ class DDSIP_Input(object):
                            self._lpfilename,
                            capabilities,
                            {'labeler' : labeler})
-        lp_file_writer.deactivate()
+        # Plugins no longer need/support calls to deactivate()
+        #lp_file_writer.deactivate()
         assert output_filename == self._lpfilename
         #print("\nModel written\n")
 
@@ -589,7 +590,7 @@ class DDSIP_Input(object):
         for scenario_tree_id, vardata in \
               iteritems(self._reference_scenario_instance.\
               _ScenarioTreeSymbolMap.bySymbol):
-            if vardata.is_expression():
+            if vardata.is_expression_type():
                 continue
             try:
                 LP_name = LP_byObject[id(vardata)]
@@ -738,9 +739,9 @@ class DDSIP_Input(object):
         for alias, obj_weakref in iteritems(LP_symbol_map.aliases):
             LP_reverse_alias[LP_byObject[id(obj_weakref())]].append(alias)
         for block in reference_instance.block_data_objects(active=True):
-            block_canonical_repn = getattr(block, "_canonical_repn", None)
-            if block_canonical_repn is None:
-                raise ValueError("Unable to find _canonical_repn ComponentMap "
+            block_repn = getattr(block, "_repn", None)
+            if block_repn is None:
+                raise ValueError("Unable to find _repn ComponentMap "
                                  "on block %s" % (block.name))
             isPiecewise = False
             if isinstance(block, (Piecewise, _PiecewiseData)):
@@ -764,7 +765,7 @@ class DDSIP_Input(object):
                 if not isPiecewise:
                     constraint_node = reference_scenario.constraintNode(
                         constraint_data,
-                        canonical_repn=block_canonical_repn.get(constraint_data),
+                        repn=block_repn.get(constraint_data),
                         instance=reference_instance)
                     stage_index = reference_scenario.node_stage_index(constraint_node)
                 else:
@@ -861,9 +862,9 @@ class DDSIP_Input(object):
                     self._num_stochastic_matrix_entries += 1
             print(("%s %s" % list(map(str,(self._reference_scenario._name, self._num_stochastic_matrix_entries)))))
 
-class ddextension(pyomo.util.plugin.SingletonPlugin):
+class ddextension(pyomo.common.plugin.SingletonPlugin):
 
-    pyomo.util.plugin.implements(phextension.IPHExtension)
+    pyomo.common.plugin.implements(phextension.IPHExtension)
 
     def __init__(self):
 
